# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sql


def upgrade(migrate_engine):
    meta = sql.MetaData()
    meta.bind = migrate_engine

    revocation_event = sql.Table('revocation_event', meta, autoload=True)
    sql.Index('ix_revocation_event_issued_before',
              revocation_event.c.issued_before).create()
    sql.Index('ix_revocation_event_project_id_issued_before',
              revocation_event.c.project_id,
              revocation_event.c.issued_before).create()
    sql.Index('ix_revocation_event_user_id_issued_before',
              revocation_event.c.user_id,
              revocation_event.c.issued_before).create()
    sql.Index('ix_revocation_event_audit_id_issued_before',
              revocation_event.c.audit_id,
              revocation_event.c.issued_before).create()
