# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystone.common import sql
from keystone.identity.mapping_backends import sql as mapping_sql


def list_id_mappings():
    """List all id_mappings for testing purposes."""
    with sql.session_for_read() as session:
        refs = session.query(mapping_sql.IDMapping).all()
        return [x.to_dict() for x in refs]
