# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


_policy_properties = {
    'blob': {
        'type': 'string'
    },
    'type': {
        'type': 'string',
        'maxLength': 255
    }
}

policy_create = {
    'type': 'object',
    'properties': _policy_properties,
    'required': ['blob', 'type'],
    'additionalProperties': True
}

policy_update = {
    'type': 'object',
    'properties': _policy_properties,
    'minProperties': 1,
    'additionalProperties': True
}
