.. -*- rst -*-

===================
OS-PKI (DEPRECATED)
===================

List revoked tokens
===================

.. rest_method:: GET /v3/auth/tokens/OS-PKI/revoked

Lists revoked PKI tokens.

Relationship:
``https://docs.openstack.org/api/openstack-identity/3/rel/tokens/OS-PKI/revoked``

Response
--------

Status Codes
~~~~~~~~~~~~

.. rest_status_code:: error status.yaml

   - 410
